Attribute VB_Name = "modSetFormOpacity"
Option Explicit
Option Compare Text
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' modSetFormOpacity
' This module contains the SetFormOpacity function that changes the
' opacity of a UserForm from fully transparent (invisible) to fully
' opaque. This required Windows 2000 or later -- it will not work
' under Windows 95, 98, or ME.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" ( _
    ByVal lpClassName As String, _
    ByVal lpWindowName As String) As Long
    
Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" ( _
    ByVal HWnd As Long, _
    ByVal nIndex As Long) As Long

Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" ( _
    ByVal HWnd As Long, _
    ByVal nIndex As Long, _
    ByVal dwNewLong As Long) As Long
    
Private Declare Function SetLayeredWindowAttributes Lib "user32" ( _
    ByVal HWnd As Long, _
    ByVal crey As Byte, _
    ByVal bAlpha As Byte, _
    ByVal dwFlags As Long) As Long

Private Declare Function FindWindowEx Lib "user32" Alias "FindWindowExA" ( _
    ByVal hWnd1 As Long, _
    ByVal hWnd2 As Long, _
    ByVal lpsz1 As String, _
    ByVal lpsz2 As String) As Long

Private Const GWL_EXSTYLE = -20
Private Const WS_EX_LAYERED = &H80000
Private Const LWA_ALPHA = &H2&
Private Const C_USERFORM_CLASSNAME = "ThunderDFrame"
Private Const C_EXCEL_DESK_CLASSNAME = "XLDesk"
Private Const C_EXCEL_WINDOW_CLASSNAME = "EXCEL7"
Private Const C_ALPHA_FULL_TRANSPARENT As Byte = 0
Private Const C_ALPHA_FULL_OPAQUE As Byte = 255


Function SetFormOpacity(WhatForm As MSForms.UserForm, _
    Opacity As Byte) As Boolean
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' SetFormOpacity
' This function sets the opacity of the UserForm referenced by the
' WhatForm parameter. Opacity specifies the opacity of the form, from
' 0 = fully transparent (invisible) to 255 = fully opaque. The function
' returns True if successful or False if an error occurred. This
' requires Windows 2000 or later -- it will not work in Windows
' 95, 98, or ME.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim UFHWnd As Long
Dim WinL As Long
Dim Res As Long

SetFormOpacity = False

UFHWnd = HWndOfUserForm(WhatForm)
If UFHWnd = 0 Then
    Exit Function
End If

WinL = GetWindowLong(UFHWnd, GWL_EXSTYLE)
If WinL = 0 Then
    Exit Function
End If

Res = SetWindowLong(UFHWnd, GWL_EXSTYLE, WinL Or WS_EX_LAYERED)
If Res = 0 Then
    Exit Function
End If

Res = SetLayeredWindowAttributes(UFHWnd, 0, Opacity, LWA_ALPHA)
If Res = 0 Then
    Exit Function
End If

SetFormOpacity = True

End Function

Function HWndOfActiveWindow() As Long
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' HWndOfActiveWindow
' This returns the HWnd of the ActiveWindow. Because the ".xls"
' extension should not be used if the Windows "hide extensions
' of known file types" setting is in effect, we call
' FindWindowEx using ActiveWindow.Caption and usinhg
' ActiveWindow.Caption with the ".xls" extension removed.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim AppHWnd As Long
Dim DeskHWnd As Long
Dim WinHWnd As Long
Dim Cap As String
Dim Pos As Long

If Application.ActiveWindow Is Nothing Then
    HWndOfActiveWindow = 0
    Exit Function
End If

AppHWnd = Application.HWnd
DeskHWnd = FindWindowEx(AppHWnd, 0&, C_EXCEL_DESK_CLASSNAME, vbNullString)
Cap = Application.ActiveWindow.Caption
WinHWnd = FindWindowEx(DeskHWnd, 0&, C_EXCEL_WINDOW_CLASSNAME, Cap)
If WinHWnd = 0 Then
    Pos = InStrRev(Cap, ".xls")
    If Pos > 0 Then
        Cap = Left(Cap, Pos - 1)
    End If
    WinHWnd = FindWindowEx(DeskHWnd, 0&, C_EXCEL_WINDOW_CLASSNAME, Cap)
End If

HWndOfActiveWindow = WinHWnd

End Function


Function HWndOfUserForm(UF As UserForm) As Long
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' HwndOfUserForm
' This returns the HWnd of the UserForm referenced by UF. The form may
' be a top level window or it may be a child of either the Application
' window or the ActiveWindow.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim AppHWnd As Long
Dim WinHWnd As Long
Dim UFHWnd As Long

UFHWnd = FindWindow(C_USERFORM_CLASSNAME, UF.Caption)
If UFHWnd <> 0 Then
    HWndOfUserForm = UFHWnd
    Exit Function
End If

AppHWnd = Application.HWnd
UFHWnd = FindWindowEx(AppHWnd, 0&, C_USERFORM_CLASSNAME, UF.Caption)
If UFHWnd <> 0 Then
    HWndOfUserForm = UFHWnd
    Exit Function
End If

WinHWnd = HWndOfActiveWindow()
If WinHWnd <> 0 Then
    UFHWnd = FindWindowEx(WinHWnd, 0&, C_USERFORM_CLASSNAME, UF.Caption)
End If

HWndOfUserForm = UFHWnd

End Function

